/*
Copyright (C) 2012 Trenz Electronic

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
IN THE SOFTWARE.
*/
/*
 * one_wire.c
 *
 *      Author: Oleksandr Kiyenko
 */
# include "one_wire.h"

#include <stdio.h>

static u8 crc8_table[] = {
        0, 94,188,226, 97, 63,221,131,194,156,126, 32,163,253, 31, 65,
      157,195, 33,127,252,162, 64, 30, 95,  1,227,189, 62, 96,130,220,
       35,125,159,193, 66, 28,254,160,225,191, 93,  3,128,222, 60, 98,
      190,224,  2, 92,223,129, 99, 61,124, 34,192,158, 29, 67,161,255,
       70, 24,250,164, 39,121,155,197,132,218, 56,102,229,187, 89,  7,
      219,133,103, 57,186,228,  6, 88, 25, 71,165,251,120, 38,196,154,
      101, 59,217,135,  4, 90,184,230,167,249, 27, 69,198,152,122, 36,
      248,166, 68, 26,153,199, 37,123, 58,100,134,216, 91,  5,231,185,
      140,210, 48,110,237,179, 81, 15, 78, 16,242,172, 47,113,147,205,
       17, 79,173,243,112, 46,204,146,211,141,111, 49,178,236, 14, 80,
      175,241, 19, 77,206,144,114, 44,109, 51,209,143, 12, 82,176,238,
       50,108,142,208, 83, 13,239,177,240,174, 76, 18,145,207, 45,115,
      202,148,118, 40,171,245, 23, 73,  8, 86,180,234,105, 55,213,139,
       87,  9,235,181, 54,104,138,212,149,203, 41,119,244,170, 72, 22,
      233,183, 85, 11,136,214, 52,106, 43,117,151,201, 74, 20,246,168,
      116, 42,200,150, 21, 75,169,247,182,232, 10, 84,215,137,107, 53};

/* Initialize 1-wire parameters
 * @param	Base address of GPIO instance
 * @param	Base address of Timer instance
 * @param	Timer Frequency
 * @return	0
 */
int one_wire_init(int gpio_addr, int timer_addr, int timer_freq){
	one_wire_gpio = (unsigned int*)gpio_addr;
	one_wire_timer = (unsigned int*)timer_addr;
	ticks_in_mks = timer_freq / 1000000;
	one_wire_gpio[GPIO_DATA_REG] = 0;
	one_wire_gpio[GPIO_TRI_REG] = 1;
	one_wire_timer[TIMER_TCSR_REG] = 0x90;
	return 0;
}

/* Stop 1-wire timer
 * @param	None
 * @return	None
 */
void one_wire_stop(){
	one_wire_gpio[GPIO_TRI_REG] = 1;
	one_wire_timer[TIMER_TCSR_REG] = 0;
}

/* Sleep required mks time
 * @param	time in mks
 * @return	0
 */
int mks_sleep(unsigned int mks){
	unsigned int cur_time, end_time;

	cur_time = one_wire_timer[TIMER_TCR_REG];
	end_time = cur_time + mks * ticks_in_mks;
	do
		cur_time = one_wire_timer[TIMER_TCR_REG];
	while (cur_time < end_time);

	return 0;
}


/* Reset 1-Wire bus
 * @param	None
 * @return	1 = Found devices, 0 = No devices found
 */
int one_wire_reset(){
	u32 present;

	one_wire_gpio[GPIO_TRI_REG] = 0;
	mks_sleep(ONE_WIRE_TRSTL);
	one_wire_gpio[GPIO_TRI_REG] = 1;
	mks_sleep(ONE_WIRE_RESP);
	present = one_wire_gpio[GPIO_DATA_REG] & 0x01;
	mks_sleep(ONE_WIRE_RSTE);
	return (present) ? 0 : 1;
}

/* Write 1 bit to 1-Wire bus
 * @param 	value == 0 -> write 0, != 0 -> write 1
 * @return	0
 */
int one_wire_write_bit(u8 btw){
	one_wire_gpio[GPIO_TRI_REG] = 0;
	mks_sleep(ONE_WIRE_TLOW);
	if(btw)
		one_wire_gpio[GPIO_TRI_REG] = 1;
	mks_sleep(ONE_WIRE_TMWR);
	one_wire_gpio[GPIO_TRI_REG] = 1;
	mks_sleep(ONE_WIRE_TREC);
	return 0;
}

/* Write byte to 1-Wire bus
 * @param 	byte to write
 * @return	0
 */
int one_wire_write_byte(u8 btw){
	int i;
	for(i = 0x01; i < 0x100; i = i << 1)
		one_wire_write_bit(btw & i);
	return 0;
}


/* Read 1 bit from 1-Wire bus
 * @param 	None
 * @return	Read result 1 or 0
 */
u8 one_wire_read_bit(){
	u32 rdbit;

	one_wire_gpio[GPIO_TRI_REG] = 0;
	mks_sleep(ONE_WIRE_TLOW);
	one_wire_gpio[GPIO_TRI_REG] = 1;
	mks_sleep(ONE_WIRE_TRDS);
	rdbit = one_wire_gpio[GPIO_DATA_REG] & 0x01;
	mks_sleep(ONE_WIRE_TMRD + ONE_WIRE_TREC);
	return (rdbit) ? 1 : 0;
}

/* Read byte from 1-Wire bus
 * @param 	None
 * @return	Read result
 */
u8 one_wire_read_byte(){
	u8 i, r;

	r = 0;
	for(i = 0; i < 8; i++){
		r = r >> 1;
		if(one_wire_read_bit())
			r |= 0x80;
	}
	return r;
}

/* Search for devices on 1-Wire bus
 * @param	device start address
 * @param	defined bits in address
 * @return	0		- No devices
 * 			255		- 1 device
 * 			1-64	- 1 device, but more than 1 can be found
 */
int one_wire_search(unsigned char* start_addr, u8 defined){
	u8 t, min_cnt, d;
	int addr_bit;

	if(!one_wire_reset()){
		return 0;	// No devices at all
	}
	one_wire_write_byte(ONE_WIRE_CMD_SEARCH_ROM);	// Start search
	min_cnt = 0;
	for(addr_bit = 0; addr_bit < 64; addr_bit++){
		d = start_addr[addr_bit >> 3] & (1 << (addr_bit & 7));
		t = one_wire_read_bit();
		t = t << 4;
		t |= one_wire_read_bit();
		switch(t){
			case 0x00:	// Both
				if((addr_bit + 1) > defined){
					one_wire_write_bit(0);	// Go to 0 branch
					start_addr[addr_bit >> 3] &= ~(1 << (addr_bit & 7));	// Store bit
					if(!min_cnt)
						min_cnt = addr_bit + 1;
				}
				else {	// Should go by defined way
					if(d){
						one_wire_write_bit(1);	// Go to 1 branch
						start_addr[addr_bit >> 3] |= (1 << (addr_bit & 7));		// Store bit
					}
					else{
						one_wire_write_bit(0);	// Go to 0 branch
						start_addr[addr_bit >> 3] &= ~(1 << (addr_bit & 7));	// Store bit
					}
				}
				break;
			case 0x01:	// Only 0s
				if((addr_bit < defined) && (d))
					return 0;
				one_wire_write_bit(0);	// Only way is 0
				start_addr[addr_bit >> 3] &= ~(1 << (addr_bit & 7));	// Store bit
				break;
			case 0x10:	// Only 1s
				if((addr_bit < defined) && !(d))
					return 0;
				one_wire_write_bit(1);	// Only way is 1
				start_addr[addr_bit >> 3] |= (1 << (addr_bit & 7));		// Store bit
				break;
			case 0x11:	// No dev
				return 0;
				break;
		}
	}
	if(min_cnt)
		return min_cnt;
	else
		return 255;
}

/* Search for devices from known family
 * use this function only if one defice from fimily present on bus.
 *
 * @param	device start address (low 8 bit is family ID)
 * @return	0		- No devices
 * 			1		- Device found
 */
int one_wire_family(unsigned char* addrp){
	if(!one_wire_reset()){
		return 0;	// No devices at all
	}
	one_wire_write_byte(ONE_WIRE_CMD_READ_ROM);
	one_wire_write_byte(addrp[0]);	// Send family ID
	addrp[1] = one_wire_read_byte();
	addrp[2] = one_wire_read_byte();
	addrp[3] = one_wire_read_byte();
	addrp[4] = one_wire_read_byte();
	addrp[5] = one_wire_read_byte();
	addrp[6] = one_wire_read_byte();
	addrp[7] = one_wire_read_byte();
	return 1;
}

/* Calculate CRC8
 * @param	old commulative CRC value
 * @param	value to add
 * @return	new CRC
 */
u8 one_wire_crc8(u8 ov, u8 av){
	return crc8_table[ov ^ av];
}
